#include <iostream>
#include <vector>
#include <cstdlib>
#include "timer.hxx"
#include "arguments.hxx"

int main(int argc, char **argv) {

  Arguments Args(argc, argv);
  size_t n = Args.Get("n", 100000000L);
  int kmax = Args.Get("kmax", 10);

  Timer T_total("global");
  
  Timer T_init("init    CPU");
  
  int i;
  int k;
  std::vector<double> u(n, 1), v(n, 2), w(n, 0);
  double a, b;

  std::cout << T_init << " s" << std::endl;

  Timer T_calcul("calcul CPU");

  for (int k=0; k<kmax; k++)
  {
#pragma omp parallel shared (u ,v ,w ,a ,b ,n) private (i)
    {
#pragma omp for
      for (i=0; i < n ; i++)
        w[i] = a * u[i] + b * v[i];
    }
  }

  std::cout << T_calcul << std::endl;

  Timer T_verif("verif. CPU");

  // verification
  for (i=0; i < n ; i++)
    if (std::abs(w[i] - a * u[i] - b * v[i]) > 1e-12) {
      std::cerr << "erreur sur la composante " << i << std::endl;
      return -1;
  }
  
  std::cout << T_verif << std::endl;

  std::cout << std::endl;
  std::cout << T_total << std::endl;
  return 0;
}
